@@ -12,7 +12,7 @@ extern "C" {
   SEXP spDynLM(SEXP Y_r, SEXP X_r, SEXP p_r, SEXP n_r, SEXP Nt_r, SEXP coordsD_r,
 	       SEXP beta0Norm_r, SEXP sigmaSqIG_r, SEXP tauSqIG_r, SEXP nuUnif_r, SEXP phiUnif_r, SEXP sigmaEtaIW_r, 
 	       SEXP betaStarting_r, SEXP phiStarting_r, SEXP sigmaSqStarting_r, SEXP tauSqStarting_r, SEXP nuStarting_r, SEXP sigmaEtaStarting_r, 
-	       SEXP phiTuning_r, SEXP nuTuning_r, SEXP covModel_r, SEXP nSamples_r, SEXP missing_r, SEXP verbose_r, SEXP nReport_r){
+	       SEXP phiTuning_r, SEXP nuTuning_r, SEXP covModel_r, SEXP nSamples_r, SEXP missing_r, SEXP getFitted_r, SEXP verbose_r, SEXP nReport_r){
     
     /*****************************************
                 Common variables
@@ -71,13 +71,16 @@ extern "C" {
 
     int nSamples = INTEGER(nSamples_r)[0];
     int *missing = INTEGER(missing_r);
+    bool getFitted = static_cast<bool>(INTEGER(getFitted_r)[0]);
     int verbose = INTEGER(verbose_r)[0];
     int nReport = INTEGER(nReport_r)[0];
 
     int nMissing = 0;
+    bool anyMissing = false;
     for(i = 0; i < Ntn; i++){
       if(missing[i] == 1){
 	nMissing++;
+	anyMissing = true;
       }
     }
   
@@ -173,14 +176,19 @@ extern "C" {
     PROTECT(beta0Samples_r = allocMatrix(REALSXP, p, nSamples)); nProtect++; 
     PROTECT(betaSamples_r = allocMatrix(REALSXP, p*Nt, nSamples)); nProtect++; 
     PROTECT(uSamples_r = allocMatrix(REALSXP, n*Nt, nSamples)); nProtect++; 
-    PROTECT(ySamples_r = allocMatrix(REALSXP, n*Nt, nSamples)); nProtect++; 
     PROTECT(sigmaEtaSamples_r = allocMatrix(REALSXP, pp, nSamples)); nProtect++; 
+
     double *thetaSamples = REAL(thetaSamples_r);
     double *beta0Samples = REAL(beta0Samples_r);
     double *betaSamples = REAL(betaSamples_r);
     double *uSamples = REAL(uSamples_r);
     double *sigmaEtaSamples = REAL(sigmaEtaSamples_r);
-    double *ySamples = REAL(ySamples_r);
+    double *ySamples = NULL;
+
+    if(getFitted){
+      PROTECT(ySamples_r = allocMatrix(REALSXP, n*Nt, nSamples)); nProtect++; 
+      ySamples = REAL(ySamples_r);
+    }
     
     /*****************************************
        Set-up MCMC alg. vars. matrices etc.
@@ -229,7 +237,7 @@ extern "C" {
     for(s = 0; s < nSamples; s++){
 
       //Sigma_eta^{-1}
-      F77_NAME(dpotrf)(lower, &p, sigmaEta, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+      F77_NAME(dpotrf)(lower, &p, sigmaEta, &p, &info); if(info != 0){error("c++ error: dpotrf1 failed\n");}
       F77_NAME(dpotri)(lower, &p, sigmaEta, &p, &info); if(info != 0){error("c++ error: dpotri failed\n");}
 
       //update Beta_0
@@ -237,7 +245,7 @@ extern "C" {
       F77_NAME(daxpy)(&pp, &one, sigma0, &incOne, tmp_pp, &incOne);
 
       //Sigma_Beta0
-      F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+      F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf2 failed\n");}
       F77_NAME(dpotri)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotri failed\n");}
 
       //mu_Beta0
@@ -245,7 +253,7 @@ extern "C" {
       F77_NAME(dsymv)(lower, &p, &one, sigmaEta, &p, beta, &incOne, &one, tmp_p, &incOne);//first column of beta is Beta_1
       F77_NAME(dsymv)(lower, &p, &one, tmp_pp, &p, tmp_p, &incOne, &zero, tmp_p2, &incOne); 
 
-     F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+     F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf3 failed\n");}
      mvrnorm(beta0, tmp_p2, tmp_pp, p);
 
      F77_NAME(dcopy)(&p, beta0, &incOne, &beta0Samples[s*p], &incOne);
@@ -253,14 +261,21 @@ extern "C" {
      ////////////////////
      //update y_t
      ////////////////////
-     for(t = 0, j = 0; t < Nt; t++){
-       F77_NAME(dgemv)(ytran, &p, &n, &one, &X[t*n*p], &p, &beta[t*p], &incOne, &zero, tmp_n3, &incOne);
-       for(i = 0; i < n; i++, j++){
-     	 if(missing[j]==1){
-     	   Y[n*t+i] = ySamples[s*Ntn+t*n+i] = rnorm(tmp_n3[i] + u[n*t+i], sqrt(theta[t*nTheta+tauSqIndx]));
-     	 }else{
-     	   ySamples[s*Ntn+t*n+i] = rnorm(tmp_n3[i] + u[n*t+i], sqrt(theta[t*nTheta+tauSqIndx]));
-     	 }
+     if(anyMissing || getFitted){
+       for(t = 0, j = 0; t < Nt; t++){
+	 F77_NAME(dgemv)(ytran, &p, &n, &one, &X[t*n*p], &p, &beta[t*p], &incOne, &zero, tmp_n3, &incOne);
+	 for(i = 0; i < n; i++, j++){
+	   if(missing[j] == 1){
+	     Y[n*t+i] = rnorm(tmp_n3[i] + u[n*t+i], sqrt(theta[t*nTheta+tauSqIndx]));
+	     if(getFitted){
+	       ySamples[s*Ntn+t*n+i] = Y[n*t+i];
+	     }
+	   }else{
+	     if(getFitted){
+	       ySamples[s*Ntn+t*n+i] = rnorm(tmp_n3[i] + u[n*t+i], sqrt(theta[t*nTheta+tauSqIndx]));
+	     }
+	   }
+	 }
        }
      }
      
@@ -278,7 +293,7 @@ extern "C" {
        tmp = 1.0/theta[t*nTheta+tauSqIndx]; 
        F77_NAME(daxpy)(&pp, &tmp, &XX[t*pp], &incOne, tmp_pp, &incOne);
    
-       F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+       F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf4 failed\n");}
        F77_NAME(dpotri)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotri failed\n");}
        
        //mu_Beta_t
@@ -300,10 +315,9 @@ extern "C" {
        }
 
        F77_NAME(dsymv)(lower, &p, &one, sigmaEta, &p, tmp_p2, &incOne, &one, tmp_p, &incOne); 
-
        F77_NAME(dsymv)(lower, &p, &one, tmp_pp, &p, tmp_p, &incOne, &zero, tmp_p2, &incOne); 
 
-       F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+       F77_NAME(dpotrf)(lower, &p, tmp_pp, &p, &info); if(info != 0){error("c++ error: dpotrf5 failed\n");}
        mvrnorm(&beta[t*p], tmp_p2, tmp_pp, p);
 
        ////////////////////
@@ -316,17 +330,13 @@ extern "C" {
        }
 	 
        spCovLT(coordsD, n, gamma, covModel, C);
-       
-       logDet = 0;
-       F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       for(i = 0; i < n; i++) logDet += 2*log(C[i*n+i]); //for phi
+    
+       F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf6 failed\n");}
        F77_NAME(dpotri)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
 
-       F77_NAME(dcopy)(&nn, C, &incOne, C3, &incOne);//save for sigma^2 and phi
-       
        F77_NAME(dgemv)(ytran, &p, &n, &one, &X[t*n*p], &p, &beta[t*p], &incOne, &zero, tmp_n3, &incOne);
 
-       if(t == 0){
+       if(t < (Nt-1)){
 
        	 //t+1
        	 gamma[0] = theta[(t+1)*nTheta+sigmaSqIndx];
@@ -337,83 +347,60 @@ extern "C" {
 	 
        	 spCovLT(coordsD, n, gamma, covModel, C2);
 	 
-       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf7 failed\n");}
        	 F77_NAME(dpotri)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
 	 
+       	 //mu
        	 F77_NAME(dsymv)(lower, &n, &one, C2, &n, &u[(t+1)*n], &incOne, &zero, tmp_n, &incOne);
-	 
-       	 F77_NAME(daxpy)(&nn, &one, C2, &incOne, C, &incOne);
-	 
-       	 for(i = 0; i < n; i++){
-       	   C[i*n+i] += tmp;
+
+       	 if(t > 0){
+       	   F77_NAME(dsymv)(lower, &n, &one, C, &n, &u[(t-1)*n], &incOne, &one, tmp_n, &incOne);
        	 }
 	 
-       	 F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       	 F77_NAME(dpotri)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
-	 	 
        	 for(i = 0; i < n; i++){
        	   tmp_n[i] += (Y[t*n+i] - tmp_n3[i])*tmp;
        	 }
 	 
-       	 F77_NAME(dsymv)(lower, &n, &one, C, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
+       	 //sigma
+       	 F77_NAME(daxpy)(&nn, &one, C, &incOne, C2, &incOne);
 	 
-       	 F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       	 mvrnorm(&u[t*n], tmp_n2, C, n);
-
-       }else if(t < (Nt-1)){
-
-       	 //t+1
-       	 gamma[0] = theta[(t+1)*nTheta+sigmaSqIndx];
-       	 gamma[1] = logitInv(theta[(t+1)*nTheta+phiIndx], phiUnif[(t+1)*2], phiUnif[(t+1)*2+1]);
-         if(covModel == "matern"){
-       	   gamma[2] = logitInv(theta[(t+1)*nTheta+nuIndx], nuUnif[(t+1)*2], nuUnif[(t+1)*2+1]);
+       	 for(i = 0; i < n; i++){
+       	   C2[i*n+i] += tmp;
        	 }
 	 
-       	 spCovLT(coordsD, n, gamma, covModel, C2);
-	 
-       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
+       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf8 failed\n");}
        	 F77_NAME(dpotri)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
-	 
-       	 F77_NAME(daxpy)(&nn, &one, C2, &incOne, C, &incOne);
-	 
-       	 for(i = 0; i < n; i++){
-       	   C[i*n+i] += tmp;
-       	 }
-	 
-       	 F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       	 F77_NAME(dpotri)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
-	 
-       	 F77_NAME(dsymv)(lower, &n, &one, C3, &n, &u[(t-1)*n], &incOne, &zero, tmp_n, &incOne);
-       	 F77_NAME(dsymv)(lower, &n, &one, C2, &n, &u[(t+1)*n], &incOne, &one, tmp_n, &incOne);
-	 
-       	 for(i = 0; i < n; i++){
-       	   tmp_n[i] += (Y[t*n+i] - tmp_n3[i])*tmp;
-       	 }
-	 
-       	 F77_NAME(dsymv)(lower, &n, &one, C, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
-	 
-       	 F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       	 mvrnorm(&u[t*n], tmp_n2, C, n);
+	 	 
+       	 //finish mu
+       	 F77_NAME(dsymv)(lower, &n, &one, C2, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
+
+       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf9 failed\n");}
+       	 mvrnorm(&u[t*n], tmp_n2, C2, n);
 
        }else{//t==Nt
-
-       	 F77_NAME(dsymv)(lower, &n, &one, C, &n, &u[(t-1)*n], &incOne, &zero, tmp_n, &incOne);
-
-       	 for(i = 0; i < n; i++){
-       	   C[i*n+i] += tmp;
-       	 }
-	 
-       	 F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       	 F77_NAME(dpotri)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
 	 
+       	 F77_NAME(dcopy)(&nn, C, &incOne, C2, &incOne);
+
+       	 //mu
+       	 F77_NAME(dsymv)(lower, &n, &one, C2, &n, &u[(t-1)*n], &incOne, &zero, tmp_n, &incOne);
+
        	 for(i = 0; i < n; i++){
        	   tmp_n[i] += (Y[t*n+i] - tmp_n3[i])*tmp;
        	 }
+
+       	 //sigma
+       	 for(i = 0; i < n; i++){
+       	   C2[i*n+i] += tmp;
+       	 }
 	 
-       	 F77_NAME(dsymv)(lower, &n, &one, C, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
+       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf10 failed\n");}
+       	 F77_NAME(dpotri)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
 	 
-       	 F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       	 mvrnorm(&u[t*n], tmp_n2, C, n);
+       	 //finish mu	 
+       	 F77_NAME(dsymv)(lower, &n, &one, C2, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
+	 
+       	 F77_NAME(dpotrf)(lower, &n, C2, &n, &info); if(info != 0){error("c++ error: dpotrf11 failed\n");}
+       	 mvrnorm(&u[t*n], tmp_n2, C2, n);
        }
  
        ////////////////////
@@ -431,14 +418,14 @@ extern "C" {
        F77_NAME(dcopy)(&n, &u[n*t], &incOne, tmp_n, &incOne);
        if(t > 0){
        	 for(i = 0; i < n; i++){
-       	   tmp_n[i] -= u[n*(t-1)+i];
+       	   tmp_n[i] = tmp_n[i] - u[n*(t-1)+i];
        	 }
        }
 
-       F77_NAME(dsymv)(lower, &n, &one, C3, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
+       F77_NAME(dsymv)(lower, &n, &one, C, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
        
        theta[t*nTheta+sigmaSqIndx] = 1.0/rgamma(sigmaSqIG[t*2]+n/2.0, 
-       						1.0/(sigmaSqIG[t*2+1]+0.5*F77_NAME(ddot)(&n, tmp_n, &incOne, tmp_n2, &incOne)));
+       						1.0/(sigmaSqIG[t*2+1]+0.5*F77_NAME(ddot)(&n, tmp_n, &incOne, tmp_n2, &incOne)*theta[t*nTheta+sigmaSqIndx]));
 
        ////////////////////
        //update phi
@@ -451,14 +438,21 @@ extern "C" {
        	 F77_NAME(dcopy)(&n, &u[n*t], &incOne, tmp_n, &incOne);
        }
 
+       //current
        gamma[0] = theta[t*nTheta+sigmaSqIndx];
        gamma[1] = logitInv(theta[t*nTheta+phiIndx], phiUnif[t*2], phiUnif[t*2+1]);
        if(covModel == "matern"){
        	 gamma[2] = logitInv(theta[t*nTheta+nuIndx], nuUnif[t*2], nuUnif[t*2+1]);
        }
 
-       //current
-       F77_NAME(dsymv)(lower, &n, &one, C3, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
+       spCovLT(coordsD, n, gamma, covModel, C);
+       
+       logDet = 0;
+       F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf12 failed\n");}
+       for(i = 0; i < n; i++) logDet += 2*log(C[i*n+i]);
+       F77_NAME(dpotri)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
+	 
+       F77_NAME(dsymv)(lower, &n, &one, C, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
        logPost = -0.5*logDet-0.5*F77_NAME(ddot)(&n, tmp_n, &incOne, tmp_n2, &incOne);
 
        logPost += log(gamma[1] - phiUnif[t*2]) + log(phiUnif[t*2+1] - gamma[1]);
@@ -469,7 +463,6 @@ extern "C" {
        //cand
        gamma[0] = theta[t*nTheta+sigmaSqIndx];
        gamma[1] = logitInv(rnorm(theta[t*nTheta+phiIndx], phiTuning[t]), phiUnif[t*2], phiUnif[t*2+1]);
-       
        if(covModel == "matern"){
        	 gamma[2] = logitInv(rnorm(theta[t*nTheta+nuIndx], nuTuning[t]), nuUnif[t*2], nuUnif[t*2+1]);
        }
@@ -477,8 +470,8 @@ extern "C" {
        spCovLT(coordsD, n, gamma, covModel, C);
        
        logDetCand = 0;
-       F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-       for(i = 0; i < n; i++) logDetCand += 2*log(C[i*n+i]); //for phi
+       F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf13 failed\n");}
+       for(i = 0; i < n; i++) logDetCand += 2*log(C[i*n+i]);
        F77_NAME(dpotri)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotri failed\n");}
        
        F77_NAME(dsymv)(lower, &n, &one, C, &n, tmp_n, &incOne, &zero, tmp_n2, &incOne);
@@ -540,7 +533,7 @@ extern "C" {
      if(verbose){
        if(status == nReport){
 	 Rprintf("Sampled: %i of %i, %3.2f%%\n", s, nSamples, 100.0*s/nSamples);
-	 Rprintf("Report interval Metrop. Acceptance rate: %3.2f%%\n", 100.0*batchAccept/(nReport*Nt));
+	 Rprintf("Report interval Mean Metrop. Acceptance rate: %3.2f%%\n", 100.0*batchAccept/(nReport*Nt));
 	 Rprintf("Overall Metrop. Acceptance rate: %3.2f%%\n", 100.0*accept/(s*Nt));
 	 Rprintf("-------------------------------------------------\n");
          #ifdef Win32
@@ -569,8 +562,11 @@ extern "C" {
     
     // //make return object
     SEXP result_r, resultName_r;
-    int nResultListObjs = 6;
-    
+    int nResultListObjs = 5;
+    if(getFitted){
+      nResultListObjs++;
+    }
+
     PROTECT(result_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
     PROTECT(resultName_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
 
@@ -590,8 +586,10 @@ extern "C" {
     SET_VECTOR_ELT(result_r, 4, thetaSamples_r);
     SET_VECTOR_ELT(resultName_r, 4, mkChar("p.theta.samples"));
 
-    SET_VECTOR_ELT(result_r, 5, ySamples_r);
-    SET_VECTOR_ELT(resultName_r, 5, mkChar("p.y.samples"));
+    if(getFitted){
+      SET_VECTOR_ELT(result_r, 5, ySamples_r);
+      SET_VECTOR_ELT(resultName_r, 5, mkChar("p.y.samples"));
+    }
 
     namesgets(result_r, resultName_r);
    
