@@ -13,7 +13,6 @@ extern "C" {
 	    SEXP betaPrior_r, SEXP betaNorm_r, SEXP sigmaSqIG_r, SEXP tauSqIG_r, SEXP nuUnif_r, SEXP phiUnif_r,
 	    SEXP phiStarting_r, SEXP sigmaSqStarting_r, SEXP tauSqStarting_r, SEXP nuStarting_r,
 	    SEXP phiTuning_r, SEXP sigmaSqTuning_r, SEXP tauSqTuning_r, SEXP nuTuning_r, 
-	    SEXP getBeta_r, SEXP rIndx_r,
 	    SEXP nugget_r, SEXP covModel_r, SEXP amcmc_r, SEXP nBatch_r, SEXP batchLength_r, SEXP acceptRate_r, SEXP verbose_r, SEXP nReport_r){
 
     /*****************************************
@@ -76,10 +75,6 @@ extern "C" {
       nuUnifa = REAL(nuUnif_r)[0]; nuUnifb = REAL(nuUnif_r)[1]; 
     }
 
-    //recover beta
-    // bool getBeta = static_cast<bool>(INTEGER(getBeta_r)[0]);
-    // int *rIndx = INTEGER(rIndx_r);
-
     bool amcmc = static_cast<bool>(INTEGER(amcmc_r)[0]);
     int nBatch = INTEGER(nBatch_r)[0];
     int batchLength = INTEGER(batchLength_r)[0];
@@ -110,10 +105,6 @@ extern "C" {
 	Rprintf("tau.sq not included in the model (i.e., no nugget model).\n\n");
       }
 
-      // if(getBeta){
-      // 	Rprintf("\nRecovering beta samples.\n\n");
-      // }
-
       Rprintf("Priors and hyperpriors:\n");
       
       if(betaPrior == "flat"){
@@ -204,24 +195,20 @@ extern "C" {
     }
 
     //return stuff  
-    SEXP samples_r, accept_r, tuning_r;//, betaSamples_r;
+    SEXP samples_r, accept_r, tuning_r;
     PROTECT(samples_r = allocMatrix(REALSXP, nParams, nSamples)); nProtect++; 
-    PROTECT(accept_r = allocMatrix(REALSXP, nParams, nBatch)); nProtect++; 
-    PROTECT(tuning_r = allocMatrix(REALSXP, nParams, nBatch)); nProtect++;
 
-    // int nRIndx = 0;
-    // if(getBeta){
-    //   for(i = 0; i < nSamples; i++){
-    // 	if(rIndx[i] == 1)
-    // 	  nRIndx++;
-    //   }
-    //   PROTECT(betaSamples_r = allocMatrix(REALSXP, p, nRIndx)); nProtect++; 
-    // }
+    if(amcmc){
+      PROTECT(accept_r = allocMatrix(REALSXP, nParams, nBatch)); nProtect++; 
+      PROTECT(tuning_r = allocMatrix(REALSXP, nParams, nBatch)); nProtect++; 
+    }else{
+      PROTECT(accept_r = allocMatrix(REALSXP, 1, floor(static_cast<double>(nSamples/nReport)))); nProtect++; 
+    }
 
     /*****************************************
        Set-up MCMC alg. vars. matrices etc.
     *****************************************/
-    int status=1, batchAccept=0;
+    int status=1, batchAccept=0, reportCnt=0;
     double logMHRatio =0, logPostCurrent = R_NegInf, logPostCand = 0, det = 0, paramsjCurrent = 0;
    
     double *paramsCurrent = (double *) R_alloc(nParams, sizeof(double));
@@ -240,29 +227,6 @@ extern "C" {
     double *tmp_p = (double *) R_alloc(p, sizeof(double));
     double *tmp_nn = NULL;
     double *Cbeta = NULL;
-      
-    //recover stuff
-    // double *B = NULL;
-    // double *bb = NULL;
-    // double *betaCInv = NULL;
-    // double *betaCInvMu = NULL;
-    // int rCnt = 0;
-
-    // if(getBeta){
-    //   B = (double *) R_alloc(pp, sizeof(double));
-    //   bb = (double *) R_alloc(p, sizeof(double));
-
-    //   if(betaPrior == "normal"){
-    // 	betaCInv = (double *) R_alloc(pp, sizeof(double));
-    // 	betaCInvMu = (double *) R_alloc(p, sizeof(double));
-
-    // 	F77_NAME(dcopy)(&pp, betaC, &incOne, betaCInv, &incOne);
-    // 	F77_NAME(dpotrf)(lower, &p, betaCInv, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-    // 	F77_NAME(dpotri)(lower, &p, betaCInv, &p, &info); if(info != 0){error("c++ error: dpotri failed\n");}
-	
-    // 	F77_NAME(dsymv)(lower, &p, &one, betaCInv, &p, betaMu, &incOne, &zero, betaCInvMu, &incOne);      
-    //   }
-    // }
 
     if(betaPrior == "normal"){
       tmp_nn = (double *) R_alloc(nn, sizeof(double));
@@ -397,44 +361,7 @@ extern "C" {
 	      accept[0]++;
 	      batchAccept++;
 	    }
-	    
-	    // //recover beta
-	    // if(getBeta && rIndx[s] == 1){
 
-	    //   if(betaPrior == "normal"){
-	    // 	F77_NAME(dpotrf)(lower, &n, C, &n, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-	    //   	F77_NAME(dcopy)(&n, Y, &incOne, vU, &incOne);
-	    //   	F77_NAME(dcopy)(&np, X, &incOne, &vU[n], &incOne);
-	    //   	F77_NAME(dtrsm)(lside, lower, ntran, nUnit, &n, &p1, &one, C, &n, vU, &n);//L[v:U] = [y:X]
-	    //   }
-
-	    //   //B
-	    //   F77_NAME(dgemm)(ytran, ntran, &p, &p, &n, &one, &vU[n], &n, &vU[n], &n, &zero, B, &p); //U'U
-	      
-	    //   if(betaPrior == "normal"){
-	    //   	for(k = 0; k < p; k++){
-	    //   	  for(l = k; l < p; l++){
-	    //   	    B[k*p+l] += betaCInv[k*p+l];
-	    //   	  }
-	    //   	}
-	    //   }
-
-	    //   F77_NAME(dpotrf)(lower, &p, B, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-	    //   F77_NAME(dpotri)(lower, &p, B, &p, &info); if(info != 0){error("c++ error: dpotri failed\n");}
-	      
-	    //   //bb
-	    //   F77_NAME(dgemv)(ytran, &n, &p, &one, &vU[n], &n, vU, &incOne, &zero, tmp_p, &incOne); //U'v
-	   	      
-	    //   if(betaPrior == "normal"){
-	    //   	for(k = 0; k < p; k++){
-	    //   	  tmp_p[k] += betaCInvMu[k];
-	    //   	}
-	    //   }
-	      
-	    //   F77_NAME(dsymv)(lower, &p, &one, B, &p, tmp_p, &incOne, &zero, bb, &incOne); 
-	    //   F77_NAME(dpotrf)(lower, &p, B, &p, &info); if(info != 0){error("c++ error: dpotrf failed\n");}
-	    // }//end update beta
-	    
 	  }else{
 	    if(amcmc){
 	      params[j] = paramsjCurrent;
@@ -453,14 +380,6 @@ extern "C" {
 	*******************************/
 	F77_NAME(dcopy)(&nParams, params, &incOne, &REAL(samples_r)[s*nParams], &incOne);
 	
-	// /******************************
-        //       Recover beta
-	// *******************************/
-	// if(getBeta && rIndx[s] == 1){
-	//   mvrnorm(&REAL(betaSamples_r)[rCnt*p], bb, B, p, false);
-	//   rCnt++;
-	// }
-
 	R_CheckUserInterrupt();
       }//end batch
       
@@ -468,7 +387,7 @@ extern "C" {
       if(amcmc){
       	for(j = 0; j < nParams; j++){
       	  REAL(accept_r)[b*nParams+j] = accept[j]/batchLength;
-      	  REAL(tuning_r)[b*nParams+j] = tuning[j];
+      	  REAL(tuning_r)[b*nParams+j] = exp(tuning[j]);
 	  
       	  if(accept[j]/batchLength > acceptRate){
       	    tuning[j] += std::min(0.01, 1.0/sqrt(static_cast<double>(b)));
@@ -480,8 +399,9 @@ extern "C" {
       }
       
       //report
-      if(verbose){
-      	if(status == nReport){
+      if(status == nReport){
+
+	if(verbose){
       	  if(amcmc){
       	    Rprintf("Batch: %i of %i, %3.2f%%\n", b+1, nBatch, 100.0*(b+1)/nBatch);
       	    Rprintf("\tparameter\tacceptance\ttuning\n");	  
@@ -502,16 +422,23 @@ extern "C" {
           #ifdef Win32
       	  R_FlushConsole();
           #endif
-      	  status = 0;
-      	  batchAccept = 0;
       	}
+
+	if(!amcmc){
+	  REAL(accept_r)[reportCnt] = 100.0*batchAccept/nReport;
+	  reportCnt++;
+	}
+
+	batchAccept = 0;
+	status = 0;
+
       }
       status++;
       
     }//end sample loop
     
     PutRNGstate();
-
+ 
     //untransform variance variables
     for(s = 0; s < nSamples; s++){
       REAL(samples_r)[s*nParams+sigmaSqIndx] = exp(REAL(samples_r)[s*nParams+sigmaSqIndx]);
@@ -528,11 +455,11 @@ extern "C" {
     
     //make return object
     SEXP result_r, resultName_r;
-    int nResultListObjs = 3;
+    int nResultListObjs = 2;
 
-    // if(getBeta){
-    //   nResultListObjs++;
-    // }
+    if(amcmc){
+      nResultListObjs++;
+    }
     
     PROTECT(result_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
     PROTECT(resultName_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
@@ -544,13 +471,10 @@ extern "C" {
     SET_VECTOR_ELT(result_r, 1, accept_r);
     SET_VECTOR_ELT(resultName_r, 1, mkChar("acceptance"));
 
-    SET_VECTOR_ELT(result_r, 2, tuning_r);
-    SET_VECTOR_ELT(resultName_r, 2, mkChar("tuning"));
-  
-    // if(getBeta){
-    //   SET_VECTOR_ELT(result_r, 3, betaSamples_r);
-    //   SET_VECTOR_ELT(resultName_r, 3, mkChar("p.beta.samples"));
-    // }
+    if(amcmc){
+      SET_VECTOR_ELT(result_r, 2, tuning_r);
+      SET_VECTOR_ELT(resultName_r, 2, mkChar("tuning"));
+    }
 
     namesgets(result_r, resultName_r);
    
